@@ -1,8 +1,9 @@
 # Version of the python-kacl package
-__version__ = "0.3.4"
+__version__ = "0.4.0"
+
+from kacl.document import KACLDocument
+from kacl.serializer import KACLMarkdownSerializer
 
-from .document import *
-from .serializer import *
 
 def load(file):
     """
@@ -10,7 +11,7 @@ def load(file):
     and produce the corresponding Python object.
     """
     doc = None
-    with open(file, 'r') as f:
+    with open(file, "r") as f:
         document = f.read()
         try:
             doc = KACLDocument.parse(document)
@@ -18,11 +19,14 @@ def load(file):
             f.close()
     return doc
 
+
 def parse(text):
     return KACLDocument.parse(text)
 
+
 def dump(document):
     return KACLMarkdownSerializer().serialize(document)
 
+
 def new():
-    return KACLDocument.init()
\ No newline at end of file
+    return KACLDocument.init()
